//  dsfid.cpp
//
//	DSF identification class
//
//	Copyright (c) 2005 N099
//
#include <windows.h>

#include "dsfid.h"


/* DSF identification class impl */
dsfid::dsfid()
{
	has_CLSID = has_NAME = listed = checked = false;
	//wcscpy(ws_cls, L"");
	//wcscpy(name, L"");
}

dsfid::dsfid(const dsfid &di)
{
	memcpy(this, &di, sizeof(dsfid));
}

dsfid::dsfid(CLSID &c)
{
	dsfid();
	setCLSID(c);
}

dsfid::dsfid(WCHAR *s_cls, WCHAR *s_name)
{
	dsfid();
	setCLSID(s_cls);
	setNAME(s_name);
}

dsfid::dsfid(CLSID &c, WCHAR *s_name)
{
	dsfid();
	setCLSID(c);
	setNAME(s_name);
}

dsfid::dsfid(WCHAR *s_name)
{
	dsfid();
	setNAME(s_name);
}

const WCHAR *dsfid::c_str_name() const
{
	return name;
}

const WCHAR *dsfid::c_str_cls() const
{
	return ws_cls;
}

bool dsfid::hasCLSID() const
{
	return has_CLSID;
}

bool dsfid::hasNAME() const
{
	return has_NAME;
}

void dsfid::setNAME(WCHAR *s_name)
{
	has_NAME = true;
	wcsncpy(name, s_name, MAX_FILTER_NAME-1);
}

void dsfid::setCLSID(CLSID &c)
{
	cls = c;
	if(CHARS_IN_GUID == StringFromGUID2(cls, ws_cls, CHARS_IN_GUID+1)){
		has_CLSID = true;
	}
}

void dsfid::setCLSID(WCHAR *s_cls)
{
	wcsncpy(ws_cls, s_cls, CHARS_IN_GUID);
	if(NOERROR == CLSIDFromString(ws_cls, &cls)){
		has_CLSID = true;
	}
}

CLSID dsfid::getCLSID(void) const
{
	return cls;
}

bool dsfid::operator==(dsfid &di)
{
	if(has_CLSID && di.hasCLSID()){
		return cls == di.cls;
	}else if(has_NAME && di.hasNAME()){
		return 0 == wcscmp(name, di.c_str_name()) ? true : false;
	}else if(has_CLSID || has_NAME){
		return false;
	}

	return true;
}

bool dsfid::operator==(const dsfid &di)
{
	dsfid d(di);
	return *this == d;
}

bool dsfid::operator>(dsfid &di)
{
	if(has_NAME && di.hasNAME()){
		return 0 < wcscmp(name, di.c_str_name()) ? true : false;
	}else if(has_CLSID && di.hasCLSID()){
		return 0 < wcscmp(ws_cls, di.c_str_cls()) ? true : false;
	}else if(has_NAME){
		return true;
	}

	return false;
}

bool dsfid::operator<(dsfid &di)
{
	if(has_NAME && di.hasNAME()){
		return 0 > wcscmp(name, di.c_str_name()) ? true : false;
	}else if(has_CLSID && di.hasCLSID()){
		return 0 > wcscmp(ws_cls, di.c_str_cls()) ? true : false;
	}else if(has_NAME){
		return false;
	}

	return true;
}

bool dsfid::operator>=(dsfid &di)
{
	return !(*this < di);
}

bool dsfid::operator<=(dsfid &di)
{
	return !(*this > di);
}

bool operator==(dsfid &di, CLSID &c)
{
	if(di.hasCLSID()){
		return di.getCLSID() == c;
	}

	return false;
}

bool operator==(dsfid &di, WCHAR *s_name)
{
	if(di.hasNAME()){
		size_t sz;
		int i;
		sz = wcslen(di.c_str_name());
		i = wcsncmp(di.c_str_name(), s_name, sz);
		return i==0 ? true : false;
	}

	return false;
}

bool operator==(CLSID &c, dsfid &di)
{
	if(di.hasCLSID()){
		return di.getCLSID() == c;
	}

	return false;
}

bool operator==(WCHAR *s_name, dsfid &di)
{
	if(di.hasNAME()){
		size_t sz;
		int i;
		sz = wcslen(di.c_str_name());
		i = wcsncmp(di.c_str_name(), s_name, sz);
		return i==0 ? true : false;
	}

	return false;
}
